/*****************************************************************************/

/*
 *      kmem.h  --  Kernel Memory Access routines.
 *
 *      Copyright (C) 2000
 *        Thomas Sailer (sailer@ife.ee.ethz.ch)
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: kmem.h,v 1.2 2000/01/11 11:07:42 tom Exp $	 
 *
 */

/*****************************************************************************/

#ifndef _KMEM_H
#define _KMEM_H

/* ---------------------------------------------------------------------- */

#include <sys/types.h>
#include <stdlib.h>

/* ---------------------------------------------------------------------- */

typedef unsigned long kaddr_t;

struct ksymtab {
	const char *name;
	kaddr_t val;
};

extern void kmem_init(struct ksymtab *symtab);
extern void *kcopy(kaddr_t addr, size_t size);
extern inline void kunuse(void *handle)
{
	free(((kaddr_t *)handle)-1);
}

extern unsigned char kreadb(kaddr_t addr);
extern unsigned short kreadw(kaddr_t addr);
extern unsigned long kreadl(kaddr_t addr);

#define offsetof(TYPE,MEMBER) ((size_t)&((TYPE *)0)->MEMBER)

extern inline kaddr_t kaddr(void *handle)
{
	kaddr_t *ptr = ((kaddr_t *)handle)-1;
	return *ptr;
}

#define kaddrof(handle,TYPE,MEMBER) (kaddr(handle)+offsetof(TYPE,MEMBER))

/* ---------------------------------------------------------------------- */

#endif /* _KMEM_H */
