/*****************************************************************************/

/*
 *      kmemread.c  --  Kernel Memory Access routines.
 *
 *      Copyright (C) 2000
 *        Thomas Sailer (sailer@ife.ee.ethz.ch)
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: kmemread.c,v 1.4 2000/01/27 14:42:26 tom Exp $	 
 *
 */

/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "kmem.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

/* ---------------------------------------------------------------------- */

static int kfd = -1;

/* ---------------------------------------------------------------------- */

void kmem_init(struct ksymtab *symtab)
{
	static struct ksymtab stab[] = {
		{ "jiffies", 0 },
		{ "high_memory", 0 },
		{ NULL, 0 }
	};
	struct ksymtab *st;
	char line[128];
	FILE *f;
	char *p;
	unsigned long sym = 0;
	unsigned int slen;
	unsigned long jiffies, memsize;
	int fd;

	for (st = symtab; st->name; st++)
		st->val = 0;
	if (!(f = fopen("/proc/ksyms", "r"))) {
		fprintf(stderr, "cannot open /proc/ksyms\n");
		exit(1);
	}
	while (fgets(line, sizeof(line), f)) {
		sym = strtoul(line, &p, 16);
		if (!p || p == line || p[0] != ' ')
			continue;
		for (st = stab ; ; st++) {
			if (!st->name) {
				if (st < stab || (char *)st > ((char *)stab)+sizeof(stab))
					break;
				else
					st = symtab;
			}
			slen = strlen(st->name);
			if (*(p+strlen(p)-1)=='\n')
			    *(p+strlen(p)-1)=0;
			//printf("<%s> <%s>\n",st->name,p+1);
			if (strncmp(p+1, st->name, slen))
				continue;
			if (p[1+slen] != 0 && p[1+slen] != ' ' && p[1+slen] != '\t' && (p[1+slen] != '_' || p[2+slen] != 'R'))
				continue;
			if (st->val) {
				fprintf(stderr, "WARNING: Symbol %s found more than once, previous 0x%x, current 0x%x\n", 
					st->name, st->val, sym);
				continue;
			}
			st->val = sym;
			printf("Symbol %s = 0x%x found\n", st->name, st->val);
			break;
		}
	}
	fclose(f);
	for (st = stab  ; st->name; st++) {
		if (!st->val) {
			fprintf(stderr, "symbol \"%s\" not found\n", st->name);
			exit(1);
		}
	}
	kfd = open("/dev/kmem", O_RDONLY);
	if (kfd == -1) {
		perror("open: /dev/kmem");
		exit(1);
	}
	jiffies = kreadl(stab[0].val);
	memsize = kreadl(stab[1].val) - 0xc0000000;
	printf("Your kernel has detected %uM %uk Bytes of memory\n"
	       "Dump at time (jiffies) %lu = 0x%x\n",
	       memsize >> 20, (memsize >> 10) & 0x3ff, jiffies, jiffies);
}

void *kcopy(kaddr_t addr, size_t size)
{
	kaddr_t *ptr;

	if (!(ptr = malloc(size+sizeof(kaddr_t)))) {
		perror("malloc");
		exit(1);
	}
	*ptr++ = addr;
	if (pread(kfd, ptr, size, addr) != size) {
		perror("pread");
		exit(1);
	}
	return ptr;
}

unsigned char kreadb(kaddr_t addr)
{
	unsigned char val;

	if (pread(kfd, &val, sizeof(val), addr) != sizeof(val)) {
		perror("pread");
		exit(1);
	}
	return val;
}

unsigned short kreadw(kaddr_t addr)
{
	unsigned short val;

	if (pread(kfd, &val, sizeof(val), addr) != sizeof(val)) {
		perror("pread");
		exit(1);
	}
	return val;
}

unsigned long kreadl(kaddr_t addr)
{
	unsigned long val;

	if (pread(kfd, &val, sizeof(val), addr) != sizeof(val)) {
		perror("pread");
		exit(1);
	}
	return val;
}

