/*****************************************************************************/

/*
 *      uhcidump.c  --  Usermode UHCI data structure dumper.
 *
 *      Copyright (C) 2000
 *        Thomas Sailer (sailer@ife.ee.ethz.ch)
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: uhcidump.c,v 1.6 2000/10/29 21:15:44 acher Exp $	 
 *
 * THIS IS NOT PORTABLE right now. It depends on the kernel configuration,
 * see eg. SMP below
 *
 */

/*****************************************************************************/

/* you need to put the following into usb-uhci.c */

#if 0
/* used by userspace UHCI data structure dumper */
puhci_t *uhci_devices = &devs;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define _XOPEN_SOURCE 500
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <linux/types.h>

#include "kmem.h"

/* ---------------------------------------------------------------------- */
/*
 * a couple of linux kernel types
 */

#undef SMP

#define deftype(nr) \
  typedef u_int##nr##_t __u##nr##; typedef int##nr##_t __s##nr##;
//deftype(32);
//deftype(16);
//deftype(8);
#undef deftype

struct list_head {
	struct list_head *next, *prev;
};

struct timer_list {
        struct timer_list *next; /* MUST be first element */
        struct timer_list *prev;
        unsigned long expires;
        unsigned long data;
        void (*function)(unsigned long);
};

#define SPINLOCK_DEBUG  1

typedef struct {
#if SMP
        volatile unsigned int lock;
#if SPINLOCK_DEBUG
        unsigned magic;
#endif
#endif
} spinlock_t;

#define PAGE_OFFSET ((kaddr_t)(0xc0000000))

#define virt_to_bus(x) (((kaddr_t)(x))-PAGE_OFFSET)
#define bus_to_virt(x) (((kaddr_t)(x))+PAGE_OFFSET)

/* ---------------------------------------------------------------------- */

/*
 * must match usb.h
 */

struct usb_device;

typedef struct
{
	unsigned int offset;
	unsigned int length;		// expected length
	unsigned int actual_length;
	unsigned int status;
} iso_packet_descriptor_t, *piso_packet_descriptor_t;

struct urb;
typedef void (*usb_complete_t)(struct urb *);

typedef struct urb
{
	void *lock;
	void *hcpriv;			// private data for host controller
	struct list_head urb_list;	// list pointer to all active urbs 
	struct urb* next;		// pointer to next URB	
	struct usb_device *dev;		// pointer to associated USB device
	unsigned int pipe;		// pipe information
	int status;			// returned status
	unsigned int transfer_flags;	// USB_DISABLE_SPD | USB_ISO_ASAP | USB_URB_EARLY_COMPLETE
	void *transfer_buffer;		// associated data buffer
	int transfer_buffer_length;	// data buffer length
	int actual_length;              // actual data buffer length	
	int bandwidth;
	unsigned char* setup_packet;	// setup packet (control only)
	//
	int start_frame;		// start frame (iso/irq only)
	int number_of_packets;		// number of packets in this request (iso/irq only)
	int interval;                   // polling interval (irq only)
	int error_count;		// number of errors in this transfer (iso only)
	int timeout;
	//
	void *context;			// context for completion routine
	usb_complete_t complete;	// pointer to completion routine
	//
	iso_packet_descriptor_t	 iso_frame_desc[0];
} urb_t, *purb_t;


/*
 * must match usb-uhci.h!
 */

typedef enum {
	TD_TYPE, QH_TYPE
} uhci_desc_type_t;

typedef struct {
	__u32 link;
	__u32 status;
	__u32 info;
	__u32 buffer;
} uhci_td_t, *puhci_td_t;

typedef struct {
	__u32 head;
	__u32 element;		/* Queue element pointer */
} uhci_qh_t, *puhci_qh_t;

typedef struct {
	union {
		uhci_td_t td;
		uhci_qh_t qh;
	} hw;
	uhci_desc_type_t type;
	struct list_head horizontal;
	struct list_head vertical;
	struct list_head desc_list;
} uhci_desc_t, *puhci_desc_t;

typedef struct {
	struct list_head desc_list;     // list pointer to all corresponding TDs/QHs associated with this request
        unsigned long started;
        urb_t *next_queued_urb;         // next queued urb for this EP
        urb_t *prev_queued_urb;
        uhci_desc_t *bottom_qh;
        uhci_desc_t *next_qh;           // next helper QH
        char use_loop;
        char flags;	                                                        
} urb_priv_t, *purb_priv_t;

struct virt_root_hub {
	int devnum;		/* Address of Root Hub endpoint */
	void *urb;
	void *int_addr;
	int send;
	int interval;
	int numports;
	int c_p_r[8];
	struct timer_list rh_int_timer;
};

typedef struct uhci {
	int irq;
	unsigned int io_addr;
	unsigned int io_size;
	unsigned int maxports;
	int running;

	int apm_state;

	struct uhci *next;	// chain of uhci device contexts

	struct list_head urb_list;	// list of all pending urbs

	spinlock_t urb_list_lock;	// lock to keep consistency 

	int unlink_urb_done;
	int avoid_bulk[2];
	
	struct usb_bus *bus;	// our bus

	__u32 *framelist;
	uhci_desc_t **iso_td;
	uhci_desc_t *int_chain[8];
	uhci_desc_t *ls_control_chain;
	uhci_desc_t *control_chain;
	uhci_desc_t *bulk_chain;
	uhci_desc_t *chain_end;
	uhci_desc_t *td1ms;
	uhci_desc_t *td32ms;
	struct list_head free_desc;
	spinlock_t qh_lock;
	spinlock_t td_lock;
	struct virt_root_hub rh;	//private data of the virtual root hub
	int loop_usage;

	struct list_head urb_unlinked;
	long timeout_check;
	int timeout_urbs;
	void * pci_dev;	
} uhci_t, *puhci_t;


#define MAKE_TD_ADDR(a) (virt_to_bus(a)&~UHCI_PTR_QH)
#define MAKE_QH_ADDR(a) (virt_to_bus(a)|UHCI_PTR_QH)
#define UHCI_GET_CURRENT_FRAME(uhci) (inw ((uhci)->io_addr + USBFRNUM))

/* ---------------------------------------------------------------------- */

#define MAXDESC 1024

struct desclist {
	unsigned int ptr;
	kaddr_t addr[MAXDESC];
};

static void dump_urb(kaddr_t urb_addr)
{
	struct urb *urb = kcopy(urb_addr, sizeof(struct urb));
	urb_priv_t *priv = kcopy((kaddr_t)urb->hcpriv, sizeof(urb_priv_t));
	static const char nullsetup[] = "\000\000\000\000\000\000\000\000";
	const char *setup = nullsetup;
	size_t size;
	unsigned int i;

	if (urb->number_of_packets > 0) {
		size = sizeof(struct urb) + urb->number_of_packets * sizeof(iso_packet_descriptor_t);
		kunuse(urb);
		urb = kcopy(urb_addr, size);
	}
	if (urb->setup_packet)
		setup = kcopy((kaddr_t)urb->setup_packet, 8);
	printf("URB: struct urb %p\n"
	       "  hcpriv = %p\n"
	       "    &desc_list = %p\n"
	       "    desc_list.next = %p\n"
	       "    desc_list.prev = %p\n"
	       "    short_control_packet = %d\n"
	       "  &urb_list = %p\n"
	       "  urb_list.next = %p\n"
	       "  urb_list.prev = %p\n"
	       "  next = %p\n"
	       "  dev = %p\n"
	       "  pipe = 0x%x\n"
	       "  status = %d (%s)\n"
	       "  transfer_flags = 0x%x\n"
	       "  transfer_buffer = %p\n"
	       "  transfer_buffer_length = %d\n"
	       "  actual_length = %d\n"
	       "  setup_packet = %p (%02x %02x %02x %02x %02x %02x %02x %02x)\n"
	       "  start_frame = %d\n"
	       "  number_of_packets = %d\n"
	       "  interval = %d\n"
	       "  error_count = %d\n"
	       "  context = %p\n"
	       "  complete = %p\n",
	       kaddr(urb), urb->hcpriv, kaddrof(priv, urb_priv_t, desc_list), 
	       priv->desc_list.next, priv->desc_list.prev, 0,
	       kaddrof(urb, struct urb, urb_list), urb->urb_list.next, urb->urb_list.prev,
	       urb->next, urb->dev, urb->pipe, urb->status, strerror(-urb->status), urb->transfer_flags, urb->transfer_buffer,
	       urb->transfer_buffer_length, urb->actual_length, 
	       urb->setup_packet, setup[0], setup[1], setup[2], setup[3], setup[4], setup[5], setup[6], setup[7], 
	       urb->start_frame, urb->number_of_packets, urb->interval, urb->error_count, urb->context, urb->complete);
	for (i = 0; i < urb->number_of_packets; i++)
		printf("  iso_frame_desc[%u]\n"
		       "    offset = %u\n"
		       "    length = %u\n"
		       "    actual_length = %u\n"
		       "    status = %u\n", i, urb->iso_frame_desc[i].offset, urb->iso_frame_desc[i].length,
		       urb->iso_frame_desc[i].actual_length, urb->iso_frame_desc[i].status);
	printf("\n");
	kunuse(priv);
	kunuse(urb);
	if (setup != nullsetup)
		kunuse((void *)setup);
}

static void dump_descriptor(kaddr_t desc_addr)
{
	uhci_desc_t *desc = kcopy(desc_addr, sizeof(uhci_desc_t));

	printf("DESC: uhci_desc_t %p\n", kaddr(desc));
	switch (desc->type) {
	case TD_TYPE:
		printf("  hw: TD\n"
		       "    link   = 0x%08x %s %s%s\n"
		       "    status = 0x%08x %sErr%d %s%s%s%s%s%s%s%s%s%sActLen %u\n"
		       "    info   = 0x%08x MaxLen %u Data%c EP%u DevNum%u PID_%02x\n"
		       "    buffer = 0x%08x\n",
		       desc->hw.td.link, (desc->hw.td.link & 2) ? "Depth" : "Breadth",
		       (desc->hw.td.link & 2) ? "QH" : "TD", (desc->hw.td.link & 1) ? " Terminate" : "",
		       desc->hw.td.status, 
		       (desc->hw.td.status & (1<<29)) ? "SPD " : "",
		       (desc->hw.td.status >> 27) & 3,
		       (desc->hw.td.status & (1<<26)) ? "LS " : "",
		       (desc->hw.td.status & (1<<25)) ? "IOS " : "",
		       (desc->hw.td.status & (1<<24)) ? "IOC " : "",
		       (desc->hw.td.status & (1<<23)) ? "Active " : "",
		       (desc->hw.td.status & (1<<22)) ? "Stalled " : "",
		       (desc->hw.td.status & (1<<21)) ? "DataBufferError " : "",
		       (desc->hw.td.status & (1<<20)) ? "Babble " : "",
		       (desc->hw.td.status & (1<<19)) ? "NAK " : "",
		       (desc->hw.td.status & (1<<18)) ? "CRC/Timeout " : "",
		       (desc->hw.td.status & (1<<17)) ? "Bitstuff " : "",
		       (desc->hw.td.status + 1) & 0x7ff,
		       desc->hw.td.info,
		       ((desc->hw.td.info >> 21) + 1) & 0x7ff,
		       ((desc->hw.td.info >> 19) & 1) + '0',
		       (desc->hw.td.info >> 15) & 15,
		       (desc->hw.td.info >> 8) & 0x7f,
		       desc->hw.td.info & 0xff,
		       desc->hw.td.buffer);
		break;

	case QH_TYPE:
		printf("  hw: QH\n"
		       "    head    = 0x%08x %s%s\n"
		       "    element = 0x%08x %s%s\n",
		       desc->hw.qh.head, (desc->hw.qh.head & 2) ? "QH" : "TD", (desc->hw.qh.head & 1) ? " Terminate" : "",
		       desc->hw.qh.element, (desc->hw.qh.element & 2) ? "QH" : "TD", (desc->hw.qh.element & 1) ? " Terminate" : "");
		break;

	default:
		printf("  hw: INVALID DESCRIPTOR TYPE %u\n", desc->type);
		break;
	}
	printf("  &horizontal = %p\n"
	       "  horizontal.next = %p\n"
	       "  horizontal.prev = %p\n"
	       "  &vertical = %p\n"
	       "  vertical.next = %p\n"
	       "  vertical.prev = %p\n"
	       "  &desc_list = %p\n"
	       "  desc_list.next = %p\n"
	       "  desc_list.prev = %p\n\n",
	       kaddrof(desc, uhci_desc_t, horizontal), desc->horizontal.next, desc->horizontal.prev,
	       kaddrof(desc, uhci_desc_t, vertical), desc->vertical.next, desc->vertical.prev,
	       kaddrof(desc, uhci_desc_t, desc_list), desc->desc_list.next, desc->desc_list.prev);
	kunuse(desc);
}

static int kaddrcompare(const void *p1, const void *p2)
{
	kaddr_t v1 = *(kaddr_t *)p1, v2 = *(kaddr_t *)p2;

	if (v1 < v2)
		return -1;
	if (v1 > v2)
		return 1;
	return 0;
}

static void compactdescs(struct desclist *dl)
{
	unsigned int ptr1 = 0, ptr2 = 0;

	qsort(&dl->addr, dl->ptr, sizeof(dl->addr[0]), kaddrcompare);
	dl->addr[ptr1++] = dl->addr[ptr2++];
	for (; ptr2 < dl->ptr; ptr2++)
		if (dl->addr[ptr2] != dl->addr[ptr1-1])
			dl->addr[ptr1++] = dl->addr[ptr2];
	dl->ptr = ptr1;
}

kaddr_t chain_end;

static void followdescs(struct desclist *dl, kaddr_t addr)
{
	kaddr_t link, elem;
	uhci_desc_type_t dt;
	int control_qh=0;

	while(1)
	{
		if (addr & 1)
			return;
		if (dl->ptr >= MAXDESC)
			compactdescs(dl);
		if (dl->ptr < MAXDESC)
			dl->addr[dl->ptr++] = bus_to_virt(addr & ~15);
		link = kreadl(bus_to_virt(addr & ~15));
		dt = kreadl(bus_to_virt(addr & ~15) + offsetof(uhci_desc_t, type));

		printf("%s", (addr & 2) ? "QH" : "TD");
		if (((addr & 2) ? QH_TYPE : TD_TYPE) != dt)
			printf("[INVALID TYPE %u]", dt);
		printf("@%08x", addr & ~15);
		if (addr & 2) {
			elem = kreadl(bus_to_virt(addr & ~15)+4);
			if (!(elem & 1)) {
				printf("(");
				followdescs(dl, elem);
				printf(")");
			}
		}
		if (link & 1)
			return;
		printf(" -> ");
		
		if ((addr&~15)==virt_to_bus(chain_end))
		{
			printf("loop to %x",link);
			return;
		}

		addr=link;
	}
//	followdescs(dl, link);
}

static void dump_device(kaddr_t puhci_addr)
{
	puhci_t puhci;
	struct desclist desclist = { 0, };
	__u32 *fl;
	unsigned int i;
	kaddr_t urb;

      	puhci = (puhci_t)kcopy(puhci_addr, sizeof(uhci_t));
	printf("UHCI: struct uhci %p\n"
	       "  irq = %d\n"
	       "  io_addr = 0x%x\n"
	       "  io_size = 0x%x\n"
	       "  maxports = %u\n"
	       "  apm_state = %d\n"
	       "  next = %p\n"
	       "  &urb_list = %p\n"
	       "    urb_list.next = %p\n"
	       "    urb_list.prev = %p\n"
#if SMP
	       "  urb_list_lock lock = 0x%x, magic = 0x%x\n"
#endif
	       "  bus = %p\n"
#if SMP
	       "  unlink_urb_lock lock = 0x%x, magic = 0x%x\n"
#endif
	       "  framelist = %p\n"
	       "  iso_td = %p\n"
	       "  int_chain[8] = %p %p %p %p %p %p %p %p\n"
	       "  control_chain = %p\n"
	       "  bulk_chain = %p\n"
	       "  chain_end = %p\n"
		" free_desc = (%p) %p : %p \n"
#if SMP
	       "  qh_lock lock = 0x%x, magic = 0x%x\n"
	       "  td_lock lock = 0x%x, magic = 0x%x\n"
#endif
	       "  virt_root_hub:\n"
	       "    devnum = 0x%x\n"
	       "    urb = %p\n"
	       "    int_addr = %p\n"
	       "    send = %d\n"
	       "    interval = %d\n"
	       "    numports = %d\n"
	       "    c_p_r[8] = %d %d %d %d %d %d %d %d\n"
	       "    rh_int_timer:\n"
	       "      next = %p\n"
	       "      prev = %p\n"
	       "      expires = 0x%x\n"
	       "      data = 0x%x\n"
	       "      function = %p\n\n",
	       kaddr(puhci), puhci->irq, puhci->io_addr, puhci->io_size, puhci->maxports,
	       puhci->apm_state, puhci->next, kaddrof(puhci, uhci_t, urb_list), puhci->urb_list.next, puhci->urb_list.prev,
#if SMP
	       puhci->urb_list_lock.lock, puhci->urb_list_lock.magic, 
#endif
	       puhci->bus, 
#if SMP
	       puhci->unlink_urb_lock.lock, puhci->unlink_urb_lock.magic,
#endif
	       puhci->framelist, puhci->iso_td,
	       puhci->int_chain[0], puhci->int_chain[1], puhci->int_chain[2], puhci->int_chain[3], 
	       puhci->int_chain[4], puhci->int_chain[5], puhci->int_chain[6], puhci->int_chain[7], 
	       puhci->control_chain, puhci->bulk_chain, puhci->chain_end,
	       kaddr(&puhci->free_desc),puhci->free_desc.next, puhci->free_desc.prev,
#if SMP
	       puhci->qh_lock.lock, puhci->qh_lock.magic, puhci->td_lock.lock, puhci->td_lock.magic, 
#endif
	       /* virtual root hub */
	       puhci->rh.devnum, puhci->rh.urb, puhci->rh.int_addr, puhci->rh.send, puhci->rh.interval, puhci->rh.numports, 
	       puhci->rh.c_p_r[0], puhci->rh.c_p_r[1], puhci->rh.c_p_r[2], puhci->rh.c_p_r[3],
	       puhci->rh.c_p_r[4], puhci->rh.c_p_r[5], puhci->rh.c_p_r[6], puhci->rh.c_p_r[7],
	       /*kaddrof(puhci, uhci_t, rh.rh_int_timer)*/NULL, puhci->rh.rh_int_timer.next, puhci->rh.rh_int_timer.prev,
	       puhci->rh.rh_int_timer.expires, puhci->rh.rh_int_timer.data, puhci->rh.rh_int_timer.function);

		chain_end=puhci->chain_end;
#if 0
#define URB_OFFSET 0   // GA
	/* dump URB list */
	for (urb = (kaddr_t)puhci->urb_list.next; urb != kaddrof(puhci, uhci_t, urb_list);
	     urb = kreadl(urb + offsetof(struct urb, urb_list.next)))
		dump_urb(urb+URB_OFFSET);
#endif
	/* dump frame list */
	printf("CHAIN END %08x\n",chain_end);
	fl = kcopy((kaddr_t)puhci->framelist, 4096);
	for (i = 0; i < 1024; i++) {
		printf("  framelist[%4u] -> ", i);
		followdescs(&desclist, fl[i]);
		printf("\n");
	}
	printf("\n");

	compactdescs(&desclist);
	for (i = 0; i < desclist.ptr; i++)
		dump_descriptor(desclist.addr[i]);

	kunuse(fl);
	kunuse(puhci);
}

/* ---------------------------------------------------------------------- */

int main(int argc, char *argv[])
{
	struct ksymtab stab[] = {
		{ "uhci_devices", 0 },
		{ NULL, 0 }
	};
	kaddr_t devs;

	printf("UHCI driver data structure dumper V" VERSION ", (C) 2000 by Thomas Sailer, sailer@ife.ee.ethz.ch\n");
	kmem_init(stab);
	if (!stab[0].val) {
		fprintf(stderr, "Symbol uhci_devices not found\n");
		exit(1);
	}
	devs = kreadl(stab[0].val);
	devs = kreadl(devs);
	while (devs) {
		dump_device(devs);
		devs = kreadl(devs + offsetof(uhci_t, next));
	}

	return 0;
}

